@@ -70,12 +70,17 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                     ('tcp:', 'session3', 'ip3:port3', '1', 'tgt3')]
         self.assertListEqual(expected, res)
 
-    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsi_session',
-                       return_value=(None, 'error'))
-    def test_get_iscsi_sessions_full_error(self, sessions_mock):
+    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsi_session')
+    def test_get_iscsi_sessions_full_stderr(self, sessions_mock):
+        iscsiadm_result = ('tcp: [session1] ip1:port1,1 tgt1 (non-flash)\n'
+                           'tcp: [session2] ip2:port2,-1 tgt2 (non-flash)\n'
+                           'tcp: [session3] ip3:port3,1 tgt3\n')
+        sessions_mock.return_value = (iscsiadm_result, 'error')
         res = self.connector._get_iscsi_sessions_full()
-        self.assertEqual([], res)
-        sessions_mock.assert_called()
+        expected = [('tcp:', 'session1', 'ip1:port1', '1', 'tgt1'),
+                    ('tcp:', 'session2', 'ip2:port2', '-1', 'tgt2'),
+                    ('tcp:', 'session3', 'ip3:port3', '1', 'tgt3')]
+        self.assertListEqual(expected, res)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
     def test_get_iscsi_sessions(self, sessions_mock):
@@ -1003,6 +1008,37 @@ Setting up iSCSI targets: unused
         self.assertListEqual(expected_cmds, actual_cmds)
         self.assertEqual(2, get_sessions_mock.call_count)
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
+    def test_connect_to_iscsi_portal_ip_case_insensitive(self,
+                                                         get_sessions_mock):
+        """Connect creating node and session."""
+        session = 'session2'
+        get_sessions_mock.side_effect = [
+            [('tcp:', 'session1', 'iP1:port1', '1', 'tgt')],
+            [('tcp:', 'session1', 'Ip1:port1', '1', 'tgt'),
+             ('tcp:', session, 'IP1:port1', '-1', 'tgt1')]
+        ]
+        with mock.patch.object(self.connector, '_execute') as exec_mock:
+            exec_mock.side_effect = [('', 'error'), ('', None),
+                                     ('', None), ('', None),
+                                     ('', None)]
+            res = self.connector._connect_to_iscsi_portal(self.CON_PROPS)
+
+        # True refers to "manual scans", since the call to update
+        # node.session.scan didn't fail they are set to manual
+        self.assertEqual((session, True), res)
+        prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
+        expected_cmds = [
+            prefix,
+            prefix + ' --interface default --op new',
+            prefix + ' --op update -n node.session.scan -v manual',
+            prefix + ' --login',
+            prefix + ' --op update -n node.startup -v automatic'
+        ]
+        actual_cmds = [' '.join(args[0]) for args in exec_mock.call_args_list]
+        self.assertListEqual(expected_cmds, actual_cmds)
+        self.assertEqual(2, get_sessions_mock.call_count)
+
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
     def test_connect_to_iscsi_portal_all_exists_chap(self, get_sessions_mock):
         """Node and session already exists and we use chap authentication."""
@@ -1291,12 +1327,13 @@ Setting up iSCSI targets: unused
         find_dm_mock.assert_not_called()
         self.assertEqual(12, connect_mock.call_count)
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('time.sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        return_value='sda')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
-    def test_connect_vol(self, connect_mock, dev_name_mock, scan_mock):
+    def test_connect_vol(self, connect_mock, dev_name_mock, scan_mock,
+                         sleep_mock):
         lscsi = self.connector._linuxscsi
         data = self._get_connect_vol_data()
         hctl = [mock.sentinel.host, mock.sentinel.channel,
@@ -1319,8 +1356,72 @@ Setting up iSCSI targets: unused
                                     mock.call(mock.sentinel.session,
                                               self.CON_PROPS['target_lun'])])
 
-        scan_mock.assert_called_once_with(*hctl)
+        scan_mock.assert_not_called()
         dev_name_mock.assert_called_once_with(mock.sentinel.session, hctl)
+        sleep_mock.assert_called_once_with(1)
+
+    @mock.patch('time.sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
+                       side_effect=(None, None, None, None, 'sda'))
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_rescan(self, connect_mock, dev_name_mock, scan_mock,
+                                sleep_mock):
+        lscsi = self.connector._linuxscsi
+        data = self._get_connect_vol_data()
+        hctl = [mock.sentinel.host, mock.sentinel.channel,
+                mock.sentinel.target, mock.sentinel.lun]
+
+        connect_mock.return_value = (mock.sentinel.session, False)
+
+        with mock.patch.object(lscsi, 'get_hctl',
+                               return_value=hctl) as hctl_mock:
+            self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(num_logins=1, stopped_threads=1,
+                        found_devices=['sda'], just_added_devices=['sda'])
+        self.assertDictEqual(expected, data)
+
+        connect_mock.assert_called_once_with(self.CON_PROPS)
+        hctl_mock.assert_called_once_with(mock.sentinel.session,
+                                          self.CON_PROPS['target_lun'])
+
+        scan_mock.assert_called_once_with(*hctl)
+        self.assertEqual(5, dev_name_mock.call_count)
+        self.assertEqual(4, sleep_mock.call_count)
+
+    @mock.patch('time.sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
+                       side_effect=(None, None, None, None, 'sda'))
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_manual(self, connect_mock, dev_name_mock, scan_mock,
+                                sleep_mock):
+        lscsi = self.connector._linuxscsi
+        data = self._get_connect_vol_data()
+        hctl = [mock.sentinel.host, mock.sentinel.channel,
+                mock.sentinel.target, mock.sentinel.lun]
+
+        # Simulate manual scan
+        connect_mock.return_value = (mock.sentinel.session, True)
+
+        with mock.patch.object(lscsi, 'get_hctl',
+                               return_value=hctl) as hctl_mock:
+            self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(num_logins=1, stopped_threads=1,
+                        found_devices=['sda'], just_added_devices=['sda'])
+        self.assertDictEqual(expected, data)
+
+        connect_mock.assert_called_once_with(self.CON_PROPS)
+        hctl_mock.assert_called_once_with(mock.sentinel.session,
+                                          self.CON_PROPS['target_lun'])
+
+        self.assertEqual(2, scan_mock.call_count)
+        self.assertEqual(5, dev_name_mock.call_count)
+        self.assertEqual(4, sleep_mock.call_count)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal',
                        return_value=(None, False))
